﻿//# sourceURL=wisej.web.ext.Uploader.startup.js

/**
 * Initializes the widget.
 *
 * This function is called when the InitScript property of
 * wisej.web.Widget changes.
 *
 * "this" refers to the container which is a wisej.web.Widget instance.
 *
 * The widget has an inner container with id = "container" that can
 * be used referring to this.container.
 *
 */

this.init = function (options) { 
    var me = this;

    this.isCancelled = false;

    //debugger;
    if (options === undefined) {
        options = this.getOptions();
    }

    if (wisej.web.DesignMode) {
        options.autoLoad = false;
    }

    this.container.classList.add("qx-button");
    //this.container.style.display = 'flex';
    //this.container.style.alignItems = 'center';

    this.controlsContainer = document.createElement("div");
    this.controlsContainer.style.cursor = 'pointer';

    this.lbl = document.createElement("label");
    this.lbl.style.cursor = 'pointer';
    this.lbl.style.display = 'inline-block';
    this.lbl.innerText = 'Custom Upload';
    this.lbl.style.width = '100%';
    this.lbl.style.height = '100%';
    this.lbl.style.textAlign = 'center';
    this.lbl.style.color = 'white';

    this.lbl.style.padding = '6px 0px';

    this.input = document.createElement("input");
    this.input.id = "Input_" + this.container.id;
    this.input.style.display = 'none';
    this.input.setAttribute("type", "file");
    this.input.addEventListener('change', function () {
        if (this.files === null) {
            me.UploadFileChanged(null,null, null);
            return;
        }
        var file = this.files[0];
        me.UploadFileChanged(file.name, file.size, file.type);
    }, false);

    this.lbl.setAttribute("for", this.input.id);

    this.controlsContainer.appendChild(this.lbl);
    this.controlsContainer.appendChild(this.input);
    this.container.appendChild(this.controlsContainer);

    window.MyUpload = this;
}

this.cancelDownload = function () {
    this.isCancelled = true;
}

this.selectFile = function () {
    this.input.click();
}

/**
 * Called when the options change. It lets the 
 * widget decide whether to update an existing
 * third-party control or to create a new one.
 */
this.update = function (options) {
    this.init(options);
}

this.startUpload = function () {
    if (this.input.files === null) {
        return;
    }
    var file = this.input.files[0];
    // This code is only for demo ...
    console.log("name : " + file.name);
    console.log("size : " + file.size);
    console.log("type : " + file.type);
    console.log("date : " + file.lastModified);
    this.UploadFile(this.input.files);
}

this.UploadFile = function (TargetFile) {
    var me = this;
    me.isCancelled = false;
    // the file object itself that we will work with
    var file = TargetFile[0];
    var fileSize = file.size;
    var fileName = file.name;

    var options = {
        binary: true,
        chunk_size: 524288,
        error_callback: function (err) {
            me.UploadFileError(err);
        },
        chunk_read_callback: function (data, offset, length, partCount) {
            var base64 = me.arrayBufferToBase64String(data);
            me.UploadFileChunk(base64, offset, length, partCount);
        },
        cancelled: function () {
            me.UploadFileCancelled();
        },
        start: function () {
            me.UploadFileStarted(fileName, fileSize);
        },
        success: function () {
            me.UploadFileSuccess();
        }
    };

    this.parseFile(file, options);
}

this.arrayBufferToBase64String = function(buffer) {
    var binaryString = '';
    var bytes = new Uint8Array(buffer);
    for (var i = 0; i < bytes.byteLength; i++) {
        binaryString += String.fromCharCode(bytes[i]);
    }

    return window.btoa(binaryString);
}

/*
 * Valid options are:
 * - chunk_read_callback: a function that accepts the read chunk
                          as its only argument. If binary option
                          is set to true, this function will receive
                          an instance of ArrayBuffer, otherwise a String
 * - error_callback:      an optional function that accepts an object of type
                          FileReader.error
 * - success:             an optional function invoked as soon as the whole file has been
                          read successfully
 * - binary:              If true chunks will be read through FileReader.readAsArrayBuffer
 *                        otherwise as FileReader.readAsText. Default is false.
 * - chunk_size:          The chunk size to be used, in bytes. Default is 64K.
 */
this.parseFile = function(file, options) {
    var opts = typeof options === 'undefined' ? {} : options;
    var fileSize = file.size;
    var chunkSize = typeof opts['chunk_size'] === 'undefined' ? 64 * 1024 : parseInt(opts['chunk_size']); // bytes
    var binary = typeof opts['binary'] === 'undefined' ? false : opts['binary'] == true;
    var offset = 0;
    var self = this; // we need a reference to the current object
    var readBlock = null;
    var chunkReadCallback = typeof opts['chunk_read_callback'] === 'function' ? opts['chunk_read_callback'] : function () { };
    var chunkErrorCallback = typeof opts['error_callback'] === 'function' ? opts['error_callback'] : function () { };
    var success = typeof opts['success'] === 'function' ? opts['success'] : function () { };
    var start = typeof opts['start'] === 'function' ? opts['start'] : function () { };
    var cancelled = typeof opts['cancelled'] === 'function' ? opts['cancelled'] : function () { };
    var partCount = 0;

    start();

    var onLoadHandler = function (evt) {
        if (evt.target.error == null) {
            var curOffset = offset;
            var curLength = evt.target.result.byteLength;
            offset += curLength;
            partCount++;
            chunkReadCallback(evt.target.result, curOffset, curLength, partCount);
        } else {
            chunkErrorCallback(evt.target.error);
            return;
        }
        if (offset >= fileSize) {
            success(file);
            return;
        }

        if (self.isCancelled) {
            cancelled();
            return;
        }

        readBlock(offset, chunkSize, file);
    }

    readBlock = function (_offset, length, _file) {
        var r = new FileReader();
        var blob = _file.slice(_offset, length + _offset);
        r.onload = onLoadHandler;
        if (binary) {
            r.readAsArrayBuffer(blob);
        } else {
            r.readAsText(blob);
        }
    }

    readBlock(offset, chunkSize, file);
}